/*
 * SBufExtras.h (C) 2008 Francesco Chemolli <kinkie@squid-cache.org>
 *
 * SQUID Web Proxy Cache          http://www.squid-cache.org/
 * ----------------------------------------------------------
 *
 *  Squid is the result of efforts by numerous individuals from
 *  the Internet community; see the CONTRIBUTORS file for full
 *  details.   Many organizations have provided support for Squid's
 *  development; see the SPONSORS file for full details.  Squid is
 *  Copyrighted (C) 2001 by the Regents of the University of
 *  California; see the COPYRIGHT file for full details.  Squid
 *  incorporates software developed and/or copyrighted by other
 *  sources; see the CREDITS file for full details.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

/*
 * This file contains various helper functions which are meant to be
 * a bridge towards SBuf integration, but which do not belong to SBuf proper,
 * such as functions shuttling to and from other data types.
 */

#include "squid.h"
#include "ipc/Messages.h"
#include "ipc/TypedMsgHdr.h"
#include "mgr/Registration.h"
#include "SBufExtras.h"
#include "StoreEntryStream.h"

SBufStatsAction::SBufStatsAction(const Mgr::CommandPointer &cmd):
        Action(cmd)
{ } //default constructor is OK for data member

SBufStatsAction::Pointer
SBufStatsAction::Create(const Mgr::CommandPointer &cmd)
{
    debugs(24, 8, "" );
    return new SBufStatsAction(cmd);
}
void
SBufStatsAction::add(const Mgr::Action& action)
{
    debugs(24, 8, "");
    data.sbdata += dynamic_cast<const SBufStatsAction&>(action).data.sbdata;
    data.mbdata += dynamic_cast<const SBufStatsAction&>(action).data.mbdata;
}

void
SBufStatsAction::collect()
{
    debugs(24, 8, "" );
    data.sbdata = SBuf::GetStats();
    data.mbdata = MemBlob::GetStats();
}

void
SBufStatsAction::dump(StoreEntry* entry)
{
    debugs(24, 8, "" );
    Must(entry != NULL);
    StoreEntryStream ses(entry);
    data.sbdata.dump(ses);
    data.mbdata.dump(ses);
}

void
SBufStatsAction::pack(Ipc::TypedMsgHdr& msg) const
{
    debugs(24, 8, "" );
    msg.setType(Ipc::mtCacheMgrResponse);
    msg.putPod(data);
}

void
SBufStatsAction::unpack(const Ipc::TypedMsgHdr& msg)
{
    debugs(24, 8, "" );
    msg.checkType(Ipc::mtCacheMgrResponse);
    msg.getPod(data);
}

class SBufStatsRegistrationHelperObject
{
public:
    SBufStatsRegistrationHelperObject() {
        Mgr::RegisterAction("sbuf", "String-Buffer statistics",
                &SBufStatsAction::Create, 0 , 1);
    }
};
static SBufStatsRegistrationHelperObject sbufstatsinstantiator;
