/*
 * SBufExtras.h (C) 2008 Francesco Chemolli <kinkie@squid-cache.org>
 *
 * SQUID Web Proxy Cache          http://www.squid-cache.org/
 * ----------------------------------------------------------
 *
 *  Squid is the result of efforts by numerous individuals from
 *  the Internet community; see the CONTRIBUTORS file for full
 *  details.   Many organizations have provided support for Squid's
 *  development; see the SPONSORS file for full details.  Squid is
 *  Copyrighted (C) 2001 by the Regents of the University of
 *  California; see the COPYRIGHT file for full details.  Squid
 *  incorporates software developed and/or copyrighted by other
 *  sources; see the CREDITS file for full details.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

/*
 * This file contains various helper functions which are meant to be
 * a bridge towards SBuf integration, but which do not belong to SBuf proper,
 * such as functions shuttling to and from other data types.
 */

#ifndef SQUID_SBUFEXTRAS_H
#define SQUID_SBUFEXTRAS_H

#include "mgr/Action.h"
#include "ipc/TypedMsgHdr.h"
#include "SBuf.h"
#include "MemBuf.h"
#include "Store.h"

class SBufStatsAction: public Mgr::Action
{
protected:
    SBufStatsAction(const Mgr::CommandPointer &cmd);
    virtual void collect();
    virtual void dump(StoreEntry* entry);

public:
    static Pointer Create(const Mgr::CommandPointer &cmd);
    virtual void add(const Mgr::Action& action);
    virtual void pack(Ipc::TypedMsgHdr& msg) const;
    virtual void unpack(const Ipc::TypedMsgHdr& msg);

private:
    struct SbufStatsActionData {
        SBufStats sbdata;
        MemBlobStats mbdata;
    } data;
};

#endif /* SQUID_SBUFEXTRAS_H */
