/*
 * SBufTokenizer.cc (C) 2008 Francesco Chemollli <kinkie@squid-cache.org>
 *
 * SQUID Web Proxy Cache          http://www.squid-cache.org/
 * ----------------------------------------------------------
 *
 *  Squid is the result of efforts by numerous individuals from
 *  the Internet community; see the CONTRIBUTORS file for full
 *  details.   Many organizations have provided support for Squid's
 *  development; see the SPONSORS file for full details.  Squid is
 *  Copyrighted (C) 2001 by the Regents of the University of
 *  California; see the COPYRIGHT file for full details.  Squid
 *  incorporates software developed and/or copyrighted by other
 *  sources; see the CREDITS file for full details.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 */

#include "squid.h"
#include "Debug.h"
#include "SBufTokenizer.h"

void SBufTokenizer::reset(const SBuf &toparse, const SBuf &delim)
{
    debugs(24, 4, "SBufTokenizer::reset()");
    _toParse = toparse;
    _delim = delim;
    _lastToken.clear();
    _lastDelim = 0;
    next();
}

SBufTokenizer& SBufTokenizer::next()
{
    debugs(24, 7, "SBufTokenizer::next()");
    SBuf::size_type tok;
    tok = _toParse.find_first_of(_delim);
    if (tok == SBuf::npos) {
        debugs(24, 8, "SBufTokenizer::token(): separator not found");
        _lastToken = _toParse;
        _toParse.clear();
        _lastDelim = '\0';
    } else {
        _lastDelim = _toParse[tok];
        _lastToken = _toParse.consume(tok);
        _toParse.consume(1); //the separator
    }
    return *this;
}

bool SBufTokenizer::atEnd() const
{
    debugs(24, 8, "result: " << (_toParse.length() == 0));
    return (_toParse.length() == 0 && _lastToken.length() == 0);
}

